<template>
	<div ref="containerRef" class="w-full flex justify-center" v-resize="resizeHandler">
		<div class="inline-block">
			<div
				:class="`space-y-[${THICK_BORDER_WIDTH}px]`"
				:style="{
					border: `${THICK_BORDER_WIDTH}px solid ${BORDER_COLOR}`,
					backgroundColor: BORDER_COLOR,
				}"
			>
				<div
					v-for="(rowChunk, index) in currentRowChunkList"
					:key="index"
					:class="`space-y-[${THIN_BORDER_WIDTH}px]`"
					:style="{
						backgroundColor: BORDER_COLOR,
					}"
				>
					<!-- 表头，和表格内容第一行 -->
					<div
						:class="`space-y-[${THICK_BORDER_WIDTH}px]`"
						:style="{
							backgroundColor: BORDER_COLOR,
						}"
					>
						<div
							class="flex"
							:class="`space-x-[${THICK_BORDER_WIDTH}px]`"
							:style="{
								backgroundColor: COL_HEADER_CELL_BG_COLOR,
							}"
						>
							<div
								v-for="(item, index) in maxColsNum"
								:key="item"
								:class="COL_HEADER_CELL_CLASS"
								:style="{
									width: `${CELL_WIDTH}px`,
									height: `${CELL_HEIGHT}px`,
									backgroundColor: COL_HEADER_CELL_BG_COLOR,
								}"
							>
								{{ index === 0 ? '' : rowChunk[index - 1]?.OTITLE }}
							</div>
						</div>
						<MonitorContent v-if="firstRow" :title="firstRow.title" :type="firstRow.id" :values="rowChunk" />
					</div>
					<!-- 剩余行 -->
					<MonitorContent v-for="(row, index) in restRows" :key="index" :title="row.title" :type="row.id" :values="rowChunk" />
				</div>
			</div>
			<el-pagination
				small
				hide-on-single-page
				v-model:current-page="pageIndex"
				v-model:page-size="pageSize"
				layout="total,prev,pager,next,jumper"
				:total="total"
				@current-change="handleCurrentChange"
			/>
		</div>
	</div>
</template>

<script setup lang="ts">
import _ from 'lodash';
import { computed, onMounted, ref } from 'vue';
import MonitorContent from './MonitorContent.vue';
import { debounce } from '/@/utils/util';

import { chatComProps } from '../../../common';
import {
	BORDER_COLOR,
	CELL_HEIGHT,
	CELL_WIDTH,
	COL_HEADER_CELL_BG_COLOR,
	COL_HEADER_CELL_CLASS,
	PAGE_HEIGHT,
	THICK_BORDER_WIDTH,
	THIN_BORDER_WIDTH,
} from './constants';
import type { Monitor, MonitorValue } from './types';

const props = defineProps(chatComProps) as {
	data: Monitor;
};
const total = props.data?.values?.length ?? 0;
const pageIndex = ref(null);
const pageSize = ref(null);

const containerRef = ref<HTMLDivElement>(null);

let maxColsNum = ref<number>(null);
const resizeEvent = ({ width, height }) => {
	// 按最大宽度算最大列数
	maxColsNum.value = Math.floor((width - THICK_BORDER_WIDTH) / (CELL_WIDTH + THICK_BORDER_WIDTH));

	const groupCount = (props.data?.rows?.length ?? 0) + 1;

	const calcMaxRowsNum = (height, extraHeight = 0) => {
		return Math.floor(
			(height - THICK_BORDER_WIDTH - extraHeight) /
				(CELL_HEIGHT * groupCount + 2 * THICK_BORDER_WIDTH + THIN_BORDER_WIDTH * (groupCount - 2))
		);
	};
	// 按最大高度算最大行数
	let rowsNum = calcMaxRowsNum(height);

	// 按最大列数铺
	const maxColsRowsNum = Math.ceil(total / maxColsNum.value);
	const isNeedPage = maxColsRowsNum > rowsNum;
	rowsNum = isNeedPage ? calcMaxRowsNum(height, PAGE_HEIGHT) : maxColsRowsNum;
	// rowsNum 行，maxColsNum列，第一列不算
	pageSize.value = isNeedPage ? rowsNum * (maxColsNum.value - 1) : total;
	pageIndex.value = 1;
	// isNeedPage 是否分页，rowsNum 行数，maxColsNum 列数，
};
const resizeHandler = debounce(resizeEvent);

const handleCurrentChange = () => {};

const firstRow = computed(() => props.data?.rows?.[0]);
const restRows = computed(() => props.data?.rows?.slice(1));
const pageChunkList = computed(() => {
	const chunkResult = _.chunk(props.data.values ?? [], pageSize.value);
	const last = chunkResult.at(-1);
	if (last) {
		const restNum = pageSize.value - last.length;
		const emptyData = _.fill(Array(restNum), {
			ONAME: '',
			OTIME: '',
			OTITLE: '',
			OTYPE: '',
		} as MonitorValue);
		const lastData = last.concat(emptyData);
		chunkResult[chunkResult.length - 1] = lastData;
	}

	return chunkResult;
});
const currentPageChunk = computed(() => {
	pageChunkList.value;
	if (pageIndex.value == null) return [];
	const pageChunk = pageChunkList.value[pageIndex.value - 1];

	return pageChunk;
});

const currentRowChunkList = computed(() => {
	if (!currentPageChunk.value || currentPageChunk.value.length === 0) return [];
	const chunkResult = _.chunk(currentPageChunk.value, maxColsNum.value - 1);
	return chunkResult;
});

// 计算最大列数
// x* cellWidth+thickBorderWidth*(x-1)+thickBorderWidth*2 <= width;

// x*(cellWidth+thickBorderWidth)+thickBorderWidth<=width;
// x<= (width-thickBorderWidth)/(cellWidth+thickBorderWidth);

// const groupCount = (TEST_DATA?.rows?.length ?? 0) + 1;
// 计算最大行数
// y * (cellHeight * groupCount) +
// 	(y - 1) * (2 * thickBorderWidth) +
// 	thickBorderWidth +
// 	thickBorderWidth * 2 +
// 	y * (groupCount - 2) * thinBorderWidth <=
// 	height;

// (cellHeight * groupCount + 2 * thickBorderWidth + thinBorderWidth(groupCount - 2) * groupCount) * y + thickBorderWidth <=height;

// y<= (height-thickBorderWidth)/(cellHeight * groupCount + 2 * thickBorderWidth + thinBorderWidth(groupCount - 2) * groupCount)

onMounted(() => {
	resizeEvent({
		width: containerRef.value.clientWidth,
		height: 0.7 * document.body.clientHeight,
	});
});
</script>
<style scoped lang="scss">
:deep(.space-y-\[2px\] > :not([hidden]) ~ :not([hidden])) {
	--tw-space-y-reverse: 0;
	margin-top: calc(2px * calc(1 - var(--tw-space-y-reverse)));
	margin-bottom: calc(2px * var(--tw-space-y-reverse));
}
:deep(.space-y-\[1px\] > :not([hidden]) ~ :not([hidden])) {
	--tw-space-y-reverse: 0;
	margin-top: calc(1px * calc(1 - var(--tw-space-y-reverse)));
	margin-bottom: calc(1px * var(--tw-space-y-reverse));
}

:deep(.space-x-\[2px\] > :not([hidden]) ~ :not([hidden])) {
	--tw-space-x-reverse: 0;
	margin-right: calc(2px * var(--tw-space-x-reverse));
	margin-left: calc(2px * calc(1 - var(--tw-space-x-reverse)));
}
:deep(.space-x-\[1px\] > :not([hidden]) ~ :not([hidden])) {
	--tw-space-x-reverse: 0;
	margin-right: calc(1px * var(--tw-space-x-reverse));
	margin-left: calc(1px * calc(1 - var(--tw-space-x-reverse)));
}
</style>
